import os
import itertools
from django.conf import settings
from django.apps import apps
from django.core.management.base import BaseCommand
from livereload import livereload_port, server as S, livereload_host
from ..getdeps import getdeps
from ..transpiler import transpile_js


class Command(BaseCommand):
    help = 'Runs a livereload server watching static files and templates.'

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            'extra',
            nargs='*',
            action='store',
            help='Extra files or directories to watch',
        )
        parser.add_argument(
            '--host',
            dest='host',
            default=livereload_host(),
            help='Host address for livereload sever.'
        )
        parser.add_argument(
            '--port',
            dest='port',
            default=livereload_port(),
            help='Listening port for livereload sever.'
        )

    def handle(self, *args, **options):
        server = S.Server()

        #  The reason not to ask the staticfile finders to list the files is
        # to get a watch when new files are added as well
        for dir in itertools.chain(
                settings.STATICFILES_DIRS,
                getattr(settings, 'TEMPLATE_DIRS', []),
                options.get('extra', []),
                args):
            server.watch(dir)
        for template in getattr(settings, 'TEMPLATES', []):
            for dir in template['DIRS']:
                server.watch(dir)
        for app_config in apps.get_app_configs():
            server.watch(os.path.join(app_config.path, 'static'))
            server.watch(os.path.join(app_config.path, 'templates'))

        if hasattr(settings, "TRANSCRYPT_MAINS"):

            for conf in settings.TRANSCRYPT_MAINS:
                path = conf["SRC_FILE"]
                sink_dir = conf["SINK_DIR"]
                fullpath = os.path.abspath(
                    os.path.join(path, "../"))
                main_file = os.path.basename(path)

                transpile_js(fullpath, main_file, sink_dir)()
                server.watch(
                    path,
                    transpile_js(fullpath,
                                 main_file,
                                 sink_dir,
                                 settings.DEBUG))

                for dep in getdeps(path):
                    server.watch(
                        dep,
                        transpile_js(fullpath, main_file, sink_dir))

        server.serve(
            host=options['host'],
            liveport=options['port'],
        )



# -*- coding: utf-8 -*-
"""Utilities related to importing modules and symbols by name."""
import imp as _imp
import importlib
import os
import sys
from contextlib import contextmanager


class NotAPackage(Exception):
    """Raised when importing a package, but it's not a package."""


@contextmanager
def cwd_in_path():
    """Context adding the current working directory to sys.path."""
    cwd = os.getcwd()
    if cwd in sys.path:
        yield
    else:
        sys.path.insert(0, cwd)
        try:
            yield cwd
        finally:
            try:
                sys.path.remove(cwd)
            except ValueError:  # pragma: no cover
                pass


def find_module(module, path=None, imp=None):
    """Version of :func:`imp.find_module` supporting dots."""
    if imp is None:
        imp = importlib.import_module
    with cwd_in_path():
        if '.' in module:
            last = None
            parts = module.split('.')
            for i, part in enumerate(parts[:-1]):
                mpart = imp('.'.join(parts[:i + 1]))
                try:
                    path = mpart.__path__
                except AttributeError:
                    raise NotAPackage(module)

                last = _imp.find_module(parts[i + 1], path)
            return last
        return _imp.find_module(module)


def import_from_cwd(module, imp=None, package=None):
    """Import module, temporarily including modules in the current directory.

    Modules located in the current directory has
    precedence over modules located in `sys.path`.
    """
    if imp is None:
        imp = importlib.import_module
    with cwd_in_path():
        return imp(module, package=package)


def reload_from_cwd(module, reloader=None):
    """Reload module (ensuring that CWD is in sys.path)."""
    if reloader is None:
        reloader = reload
    with cwd_in_path():
        return reloader(module)


def module_file(module):
    """Return the correct original file name of a module."""
    name = module.__file__
    return name[:-1] if name.endswith('.pyc') else name


def load_extension_class_names(namespace):
    try:
        from pkg_resources import iter_entry_points
    except ImportError:  # pragma: no cover
        return

    for ep in iter_entry_points(namespace):
        yield ep.name, ':'.join([ep.module_name, ep.attrs[0]])



import ast
import os
import glob
import site
from .imports import find_module
from .imports import NotAPackage
from org.transcrypt.utils import commandArgs


commandArgs.source = None


class ImportLister(ast.NodeVisitor):

    fileList = []
    moduleList = []

    def _process_names(self, names, module=None):
        for item in names:
            item_module = item.name
            if module is not None:
                item_module = module + "." + item_module
            self.moduleList.append(item_module)

    def find_modules(self, prefix=[]):
        """
        Returns a list of paths to dependencies
        """
        for pack in set(self.moduleList):
            path = None
            try:
                _, path, _ = find_module(pack)
            except (NotAPackage, ImportError):
                try:
                    _, path, _ = find_module(".".join(pack.split(".")[:-1]))
                except (NotAPackage, ImportError):
                    pass

            if path is not None:
                if os.path.isfile(path):
                    self.fileList.append(path)
                else:
                    self.fileList.append(path + "/*.py")

    def visit_Import(self, node):
        self._process_names(node.names)
        self.generic_visit(node)

    def visit_ImportFrom(self, node):
        self._process_names(node.names, node.module)
        self.generic_visit(node)


def _get_filename_deps(filename):
    code = ""
    for file in glob.glob(filename):
        with open(file, encoding='utf-8') as f:
            code = f.read()
        tree = ast.parse(code)
        path = list(os.path.split(file)[:-1])
        importLister = ImportLister()
        importLister.generic_visit(tree)
        importLister.find_modules(path)

    return importLister.fileList


def getdeps(filename):
    # Initial run
    sitepackages_dir = site.getsitepackages()

    deps = set(_get_filename_deps(filename))
    deps_count = 0
    while deps_count != len(deps):
        deps_count = len(deps)
        for dep in deps:
            ndeps = _get_filename_deps(dep)
            for dir in sitepackages_dir:
                ndeps = [path for path in ndeps if not path.startswith(dir)]
            deps = deps | set(ndeps)

    return list(deps)



import os
import glob
import shutil

DEBUG_OPTIONS = ['--map']
PRODUCTION_OPTIONS = ['-b']


def transpile_js(path, file, sink, debug=True):
    os.chdir(path)
    js_path = os.path.join(path, "__javascript__")

    def inner_transpile():
        if debug:
            transpiler_str = " ".join(["transcrypt"] +
                                      DEBUG_OPTIONS +
                                      [file])
        else:
            transpiler_str = " ".join(["transcrypt"] +
                                      PRODUCTION_OPTIONS +
                                      [file])
        os.system(transpiler_str)
        files = glob.glob(os.path.join(js_path, '*.js'),
                          recursive=False) +\
            glob.glob(os.path.join(js_path, '**/*.map'),
                      recursive=True)

        for filepath in files:
            if (os.path.isfile(filepath)):

                filepath_postfix = filepath.replace(js_path, "")
                if os.path.isabs(filepath_postfix):
                    filepath_postfix = filepath_postfix[1:]
                dst = os.path.join(
                    sink,
                    filepath_postfix)
                if not os.path.exists(os.path.dirname(dst)):
                    os.makedirs(os.path.dirname(dst))
                print("Copying file: " + filepath + " -> " + dst)
                shutil.copy(
                    src=filepath,
                    dst=dst)

        print("\033[92m[Success] \033[0m")

    return inner_transpile



# -*- coding: utf-8 -*-
from distutils.core import setup

setup(
    name='django-transcrypt-exp',
    version='0.0.1-4',
    author=u'Morten B. Rasmussen',
    author_email='mbr@mr-systems.dk',
    packages=[
        'django_transcrypt',
        'django_transcrypt.management',
        'django_transcrypt.management.commands'],
    include_package_data=True,
    url='https://bitbucket.org/cruise/django_transcrypt',
    license='MIT, see LICENSE.txt',
    description='Adds livereload facility together with transcrypt python to' +
                ' javascript transpiling',
    long_description=open('README.txt').read(),
    zip_safe=False,
    install_requires=[
        'django-livereload-server',
        'Transcrypt'],
)



from django.core.management.base import BaseCommand
from django.conf import settings
from ..transpiler import transpile_js

import os


class Command(BaseCommand):
    help = 'Transpiles python files to js according to the settings'

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            '--debug',
            dest='debug',
            action='store_true',
            help='Use debug transpiler options',
        )

    def handle(self, *args, **options):
        if hasattr(settings, "TRANSCRYPT_MAINS"):

            for conf in settings.TRANSCRYPT_MAINS:
                path = conf["SRC_FILE"]
                sink_dir = conf["SINK_DIR"]
                fullpath = os.path.abspath(
                    os.path.join(path, "../"))
                main_file = os.path.basename(path)

                transpile_js(fullpath, main_file, sink_dir)()
